/*
 * Copyright (c) 2011 Fabrice Bellard
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <stddef.h>
#include <stdarg.h>

#ifdef TEST
#include <stdlib.h>
#include <inttypes.h>
#else

typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;
#if defined (__x86_64__)
typedef unsigned long uint64_t;
#else
typedef unsigned long long uint64_t;
#endif

typedef signed char int8_t;
typedef signed short int16_t;
typedef signed int int32_t;
#if defined (__x86_64__)
typedef signed long int64_t;
#else
typedef signed long long int64_t;
#endif

#if defined (__x86_64__)
typedef uint64_t uintptr_t;
typedef int64_t intptr_t;
#else
typedef uint32_t uintptr_t;
typedef int32_t intptr_t;
#endif

#ifndef NULL
#define NULL 0
#endif

void *memset(void *d1, int val, size_t len);
void *memcpy(void *d1, const void *s1, size_t len);
void *memmove(void *d1, const void *s1, size_t len);
size_t strlen(const char *s);
char *strcpy(char *d, char *s);

void putchar(int c);
void __attribute__((noreturn)) exit(int val);
void *malloc(size_t size);
void free(void *ptr);
#endif /* !TEST */

void put_string(const char *s);
void put_int(unsigned int v);
int vsnprintf(char *buf, int buflen, const char *fmt, va_list args);
void __attribute((format (printf, 1, 2))) printf(const char *fmt, ...);

#define assert(a) \
    do {                                                            \
        if (!(a)) {                                                 \
            printf("%s:%d:" #a "\n", __FILE__, __LINE__);           \
            exit(1);                                                \
        }                                                           \
    } while (0)

